/* Nihongo Benkyo
* Copyright (C) 2005 Mathieu Blondel
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ruby.h>
#include "nihongobenkyo_import.h"

static VALUE rb_mNihongoBenkyo;
static VALUE rb_mNihongoBenkyoImport;

static VALUE
edict(VALUE module, VALUE db_file, VALUE dic_file, VALUE dic_name)
{
    Check_Type(db_file, T_STRING);
    Check_Type(dic_file, T_STRING);
    Check_Type(dic_name, T_STRING);
    
    edict_import(StringValuePtr(db_file),
                 StringValuePtr(dic_file),   
                 StringValuePtr(dic_name)
                 );
                 
    return Qtrue;
}

static VALUE
jmnedict(VALUE module, VALUE db_file, VALUE dic_file, VALUE dic_name)
{
    Check_Type(db_file, T_STRING);
    Check_Type(dic_file, T_STRING);
    Check_Type(dic_name, T_STRING);
    
    jmnedict_import(StringValuePtr(db_file),
                    StringValuePtr(dic_file),   
                    StringValuePtr(dic_name)
                   );
                 
    return Qtrue;
}

static VALUE
kanjidic2(int argc, VALUE *argv, VALUE module)
{
    int i;
    struct kanjidic2_langs langs = {0,0,0};
    
    /* At least, db_file, dic_file, dic_name and one lang */
    if(argc < 4) {
        rb_raise(rb_eStandardError, "kanjidic2 needs at least 4 arguments");
    }
    
    /* Everything must be a string */
    for(i=0; i < argc; i++) {
        Check_Type(argv[i], T_STRING);
    }
    
    /* Fill the structure */
    for(i=3; i < argc; i++) {
        if(strcmp(StringValuePtr(argv[i]), "en") == 0)langs.en = 1;
        if(strcmp(StringValuePtr(argv[i]), "es") == 0)langs.es = 1;
        if(strcmp(StringValuePtr(argv[i]), "pt") == 0)langs.pt = 1;
    }
    
    kanjidic2_import(StringValuePtr(argv[0]), StringValuePtr(argv[1]), StringValuePtr(argv[2]), langs);
    
    return Qtrue;
}

static VALUE
jmdict(int argc, VALUE *argv, VALUE module)
{
    int i;
    struct jmdict_langs langs = {0,0,0,0,0};
    
    /* At least, db_file, dic_file, dic_name and one lang */
    if(argc < 4) {
        rb_raise(rb_eStandardError, "jmdict needs at least 4 arguments");
    }
    
    /* Everything must be a string */
    for(i=0; i < argc; i++) {
        Check_Type(argv[i], T_STRING);
    }
    
    /* Fill the structure */
    for(i=3; i < argc; i++) {
        if(strcmp(StringValuePtr(argv[i]), "en") == 0)langs.en = 1;
        if(strcmp(StringValuePtr(argv[i]), "fr") == 0)langs.fr = 1;
        if(strcmp(StringValuePtr(argv[i]), "de") == 0)langs.de = 1;
        if(strcmp(StringValuePtr(argv[i]), "nl") == 0)langs.nl = 1;
        if(strcmp(StringValuePtr(argv[i]), "ru") == 0)langs.ru = 1;
    }
    
    jmdict_import(StringValuePtr(argv[0]), StringValuePtr(argv[1]), StringValuePtr(argv[2]), langs);
    
    return Qtrue;
}

void
Init_import()
{
    /* Modules */
    rb_mNihongoBenkyo = rb_define_module("NihongoBenkyo");
    rb_mNihongoBenkyoImport = rb_define_module_under(rb_mNihongoBenkyo, "Import");
    
    /* Exceptions */
    rb_eDicAlreadyExistsException = rb_define_class_under(rb_mNihongoBenkyoImport,
                                                           "DicAlreadyExistsException",
                                                           rb_eStandardError
                                                         );
                                                           
    rb_eDicTypeException = rb_define_class_under(rb_mNihongoBenkyoImport,
                                                 "DicTypeException",
                                                 rb_eStandardError
                                                );
                                                 
    /* Methods */
    
    rb_define_module_function(rb_mNihongoBenkyoImport,
                              "edict",
                              edict,
                              3
                             );
                             
    rb_define_module_function(rb_mNihongoBenkyoImport,
                              "jmdict",
                              jmdict,
                              -1
                             );
                             
    rb_define_module_function(rb_mNihongoBenkyoImport,
                              "jmnedict",
                              jmnedict,
                              3
                             );
                             
    rb_define_module_function(rb_mNihongoBenkyoImport,
                              "kanjidic2",
                              kanjidic2,
                              -1
                             );
}   

