/* Nihongo Benkyo
* Copyright (C) 2005 Mathieu Blondel
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <sqlite3.h>
#include <ruby.h>
#include <libxml/xmlreader.h>

/* Types of dic */

#define WORD_DIC 0
#define KANJI_DIC 1

/* Ruby objects */

VALUE rb_eDicAlreadyExistsException;
VALUE rb_eDicTypeException;

/* EDICT */

void edict_import(char *db_file, char *dic_file, char *dic_name);

/* Used for xml files */

enum textNodeName {
    NONE = 0,
    KEB = 1,
    REB = 2,
    GLOSS = 3,
    TRANS_DET = 4,
    KE_INF = 5,
    KE_PRI = 6,
    RE_NOKANJI = 7,
    RE_INF = 8,
    RE_PRI = 9,
    RE_RESTR = 10,
    LANG = 11,
    DIAL = 12,
    ETYM = 13,
    STAGK = 14,
    STAGR = 15,
    XREF = 16,
    ANT = 17,
    POS = 18,
    FIELD = 19,
    MISC = 20,
    S_INF = 21,
    NAME_TYPE = 22,
    LITERAL = 23,
    READING = 24,
    MEANING = 25,
    GRADE = 26,
    STROKE_COUNT = 27,
    FREQ = 28
};

enum metadata_name {
    WORD_KANJI_INFO = 1,
    WORD_KANJI_SOURCE = 2,
    WORD_KANJI_PRIORITY = 3,
    WORD_READING_NO_KANJI = 4,
    WORD_READING_INFO = 5,
    WORD_READING_SOURCE = 6,
    WORD_READING_PRIORITY = 7,
    WORD_READING_RESTRICTION = 8,
    WORD_ENTRY_ORIGIN_LANG = 9,
    WORD_ENTRY_DIALECT = 10,
    WORD_ENTRY_ETYMOLOGY = 11,
    WORD_SENSE_KANJI_RESTRICTION = 12,
    WORD_SENSE_READING_RESTRICTION = 13,
    WORD_SENSE_REFERENCE = 14,
    WORD_SENSE_ANTONYM = 15,
    WORD_SENSE_PART_OF_SPEECH = 16,
    WORD_SENSE_FIELD = 17,
    WORD_SENSE_MISC = 18,
    WORD_SENSE_INFO = 19,
    WORD_SENSE_NAME_TYPE = 20,
    KANJI_STROKE_COUNT = 21,
    KANJI_GRADE = 22,
    KANJI_FREQUENCE = 23
};

void stream_file(const char *filename, void (*process_node_ptr) (xmlTextReaderPtr));
int start_element(xmlTextReaderPtr reader, const xmlChar *name, char *element);
int end_element(xmlTextReaderPtr reader, const xmlChar *name, char *element);

/* JMDict */

struct jmdict_langs {
    int en;
    int fr;
    int de;
    int nl;
    int ru;
};

void jmdict_import(char *db_file, char *dic_file, char *dic_name, struct jmdict_langs langs);

/* JMnedict */

void jmnedict_import(char *db_file, char *dic_file, char *dic_name);

/* Kanjidic2 */

struct kanjidic2_langs {
    int en;
    int pt;
    int es;
};

void kanjidic2_import(char *db_file, char* dic_file, char *dic_name, struct kanjidic2_langs langs);

/* Database */

int database_open(sqlite3 **db, char *database_file);
int database_close(sqlite3 *db);
int database_add_dictionary(sqlite3 *db, char *dic_name, int dic_type, int position);
int database_add_entry(sqlite3 *db, int dictionary_id, int entry_id);
int database_add_kanji(sqlite3 *db, int entry_id, int kanji_id, char *kanji);
int database_add_reading(sqlite3 *db, int entry_id, int reading_id, char *type, char *reading);
int database_add_translation(sqlite3 *db, int entry_id, int translation_id, char *translation,
                             int sense_id, char *language); 
int database_add_metadata_value(sqlite3 *db, int matching_id, int metadata_id, char *metadata_value);
int database_get_last_entry_id(sqlite3 *db);
int database_get_last_kanji_id(sqlite3 *db);
int database_get_last_reading_id(sqlite3 *db);
int database_get_last_translation_id(sqlite3 *db);
int database_get_last_sense_id(sqlite3 *db);
int database_get_last_dictionary_id(sqlite3 *db);
int database_get_dictionary_id(sqlite3 *db, char *dic_name);
int database_dictionary_exists(sqlite3 *db, char *dic_name);
int database_execute(sqlite3 *db, char *query);
void database_create_indexes(sqlite3 *db);
void database_drop_indexes(sqlite3 *db);
int database_has_indexes(sqlite3 *db);
