/* Nihongo Benkyo
* Copyright (C) 2005 Mathieu Blondel
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <libxml/xmlreader.h>
#include "nihongobenkyo_import.h"

int
start_element(xmlTextReaderPtr reader, const xmlChar *name, char *element) {
    return (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT 
            && xmlStrcmp(name, (xmlChar *) element) == 0);
}

int
end_element(xmlTextReaderPtr reader, const xmlChar *name, char *element) {
    if(element != NULL) {
        return (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT 
                && xmlStrcmp(name, (xmlChar *) element) == 0);
    }
    else {
        return (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT);
    }
}

void
stream_file(const char *filename, void (*process_node_ptr) (xmlTextReaderPtr)) {
    xmlTextReaderPtr reader;
    int ret;

    reader = xmlReaderForFile(filename, NULL, XML_PARSE_NOENT);
    if (reader != NULL) {
        ret = xmlTextReaderRead(reader);
        while (ret == 1) {
            (*process_node_ptr)(reader);
            ret = xmlTextReaderRead(reader);
        }
        xmlFreeTextReader(reader);
        if (ret != 0) {
            rb_raise (rb_eDicTypeException, "Can't parse xml file");
        }
    } else {
        rb_raise (rb_eIOError, "Could not open dic file"); 
    }
    
    xmlCleanupParser();
}
